//
//	Leaky Integrate-and-Fire neuron model
//				coded by A. Ishihara
//
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

int main()
{
	// Parameters of the model
	double C = 1.0;	// membrane capacitance [nF]
	double R = 40;	// membrane resistance [M ohms]

	// external input
	double I = 1.0; // driving current [nA]
	
	// Variables of the model
	double V = 0.0;			// membrane potential [mV]
	double theta = 15.0;	// threshold for spiking [mV]
	double Vr = 0.0;		// resting potential [mV]
	double Vsp = 50.0;		// maximal potential of a spike
	int abs_ref = 5;		// absolute refractory period
	
	int tstop = 200;		// end of time
	int c_ref = 0;			// counter for refractory period
	double dV;				// dV/dt
	double taum = R*C;		// time constant of leaky integrator

	// output to display of initial value
	fprintf( stdout, "%f, %f, %f\n", 0.0, I, V );

	// the model simulation is implemented 
	// by using h = 1.0 ms step size, Euler method
	double h = 1.0;			// calculation step [ms]
	for( int i = 0; i <= tstop; i++ ){
		if( c_ref == 0 ){
			// calculation of membrane potential
			dV = (-V + R*I)/taum;
			V = V + h*dV;
		} else {
			// response in refractory period
			c_ref--;
			V = Vr;
		}
		// spiking
		if( V > theta ){
			V = Vsp;
			c_ref = abs_ref;
		}
		// output to display 
		fprintf( stdout, "%f, %f, %f\n", h*i, I, V );
	}

	// end
	return 0;
}
